/* Warm parchment-tinted background -- slightly off-white towards
   brownish-yellow, easier on the eyes than pure white. */
body {
    background-color: #f5f0e8;
}

div.body {
    background-color: #faf6ef;
}

div.sphinxsidebar {
    background-color: #efe9de;
}

/* Pin the layout to the left instead of centering on wide screens. */
div.document {
    margin: 0;
}

@media screen and (max-width: 940px) {
    div.sphinxsidebar {
        background: #efe9de;
        color: #333;
    }

    div.sphinxsidebar h3, div.sphinxsidebar h4, div.sphinxsidebar p,
    div.sphinxsidebar h3 a {
        color: #333;
    }

    div.sphinxsidebar a {
        color: #444;
    }
}
